from .base import BaseResource


class Movies(BaseResource):
    """Interact with The Movie Database Movies endpoints

    See: http://docs.themoviedb.apiary.io/#movies
    """
    BASE_PATH = 'movie/'
    URLS = {
        'info': '{id}',
        'alternative_titles': '{id}/alternative_titles',
        'credits': '{id}/credits',
        'images': '{id}/images',
        'keywords': '{id}/keywords',
        'releases': '{id}/releases',
        'trailers': '{id}/trailers',
        'translations': '{id}/translations',
        'similar_movies': '{id}/similar_movies',
        'reviews': '{id}/reviews',
        'lists': '{id}/lists',
        'changes': '{id}/changes',
        'latest': 'latest',
        'upcoming': 'upcoming',
        'now_playing': 'now_playing',
        'popular': 'popular',
        'top_rated': 'top_rated',
        'account_states': '{id}/account_states',
        'rating': '{id}/rating',
    }

    async def info(self, movie_id, **kwargs):
        """Get the basic movie information for a specific movie id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D
        """
        path = self._get_path('info').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def alternative_titles(self, movie_id, **kwargs):
        """Get the alternative titles for a specific movie id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Falternative_titles
        """
        path = self._get_path('alternative_titles').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def credits(self, movie_id, **kwargs):
        """Get the cast and crew information for a specific movie id.

        TMDB docs: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Fcredits

        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('credits').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def images(self, movie_id, **kwargs):
        """Get the images (posters and backdrops) for a specific movie id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Fimages

        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param include_image_language: (optional) Comma separated ISO 639-1 codes,
                                       e.g. 'en, es'.
        :param append_to_response: (optional) Any Movies method names, comma
                                   separated, e.g. 'credits, images'.
        """
        path = self._get_path('images').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def keywords(self, movie_id, **kwargs):
        """Get the plot keywords for a specific movie id.

        TMDB docs: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Fkeywords

        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('keywords').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def releases(self, movie_id, **kwargs):
        """Get the release date by country for a specific movie id.

        TMDB docs: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Freleases

        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('releases').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def trailers(self, movie_id, **kwargs):
        """Get the trailers for a specific movie id.

        TMDB docs: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Ftrailers

        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('trailers').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def translations(self, movie_id, **kwargs):
        """Get the translations for a specific movie id.

        TMDB docs: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Ftranslations

        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('translations').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def similar_movies(self, movie_id, **kwargs):
        """Get the similar movies for a specific movie id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Fsimilar_movies

        :param page: (optional) Result page number, e.g. 3.
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('similar_movies').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def reviews(self, movie_id, **kwargs):
        """Get the reviews for a particular movie id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Freviews

        :param page: (optional) Result page number, e.g. 3.
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('reviews').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def lists(self, movie_id, **kwargs):
        """Get the lists that the movie belongs to.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Flists

        :param page: (optional) Result page number, e.g. 3.
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_path('lists').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def changes(self, movie_id, **kwargs):
        """Get the changes for a specific movie id.

        Changes are grouped by key, and ordered by date in descending order. By
        default, only the last 24 hours of changes are returned. The maximum number
        of days that can be returned in a single request is 14. The language is
        present on fields that are translatable.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Fchanges

        :param start_date: (optional) A string in the format 'YYYY-MM-DD'
        :param end_date: (optional) Same as start_date
        """
        path = self._get_path('changes').format(id=movie_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def latest(self, **kwargs):
        """Get the latest movie id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2Flatest
        """
        path = self._get_path('latest')

        response = await self._get(path, **kwargs)
        
        return response

    async def upcoming(self, **kwargs):
        """Get the list of upcoming movies.

        This list refreshes every day. The maximum number of items this list will include is 100.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2Fupcoming
        """
        path = self._get_path('upcoming')

        response = await self._get(path, **kwargs)
        
        return response

    async def now_playing(self, **kwargs):
        """Get the list of movies playing in theatres.

        This list refreshes every day. The maximum number of items this list will include is 100.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2Fnow_playing
        """
        path = self._get_path('now_playing')

        response = await self._get(path, **kwargs)
        
        return response

    async def popular(self, **kwargs):
        """Get the list of popular movies on The Movie Database.

        This list refreshes every day.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2Fpopular
        """
        path = self._get_path('popular')

        response = await self._get(path, **kwargs)
        
        return response

    async def top_rated(self, **kwargs):
        """Get the list of top rated movies.

        This list refreshes every day. This list only includes movies that have
        10 or more votes.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2Ftop_rated
        """
        path = self._get_path('top_rated')

        response = await self._get(path, **kwargs)
        
        return response

    async def account_states(self, movie_id, session_id, **kwargs):
        """Get status of whether or not the movie has been rated or added to favourite or watch lists.

        A `valid session <https://www.themoviedb.org/documentation/api/sessions>`_ id is required.
        """
        path = self._get_path('account_states').format(id=movie_id)
        kwargs.update({'session_id': session_id})

        response = await self._get(path, **kwargs)
        
        return response

    async def rating(self, movie_id, rating, session_id=None, guest_session_id=None, **kwargs):
        """Rate a movie.

        A `valid session <https://www.themoviedb.org/documentation/api/sessions>`_
        id or guest session id is required.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D%2Flists
        """
        path = self._get_path('rating').format(id=movie_id)
        if session_id is None and guest_session_id is None:
            raise ValueError("Either 'session_id' or 'guest_session_id' keyword argument is required")
        payload = {'value': rating}
        if session_id:
            kwargs.update({'session_id': session_id})
        else:
            kwargs.update({'guest_session_id': guest_session_id})

        response = self._post(path, kwargs, payload)
        
        return response


class Collections(BaseResource):
    """Get information about movie collections

    See: http://docs.themoviedb.apiary.io/#collections
    """
    BASE_PATH = 'collection/'
    URLS = {
        'info': '{id}',
        'images': '{id}/images',
    }

    async def info(self, collection_id, **kwargs):
        """Get the basic collection information for a specific collection id.

        You can get the ID needed for this method by making a :py:meth:`Movies.info`
        request and paying attention to the `belongs_to_collection` hash.

        Movie parts are not sorted in any particular order. If you would like
        to sort them yourself you can use the provided `release_date`.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fcollection%2F%7Bid%7D
        """
        path = self._get_path('info').format(id=collection_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def images(self, collection_id, **kwargs):
        """Get all of the images for a particular collection by collection id.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fcollection%2F%7Bid%7D%2Fimages
        """
        path = self._get_path('images').format(id=collection_id)

        response = await self._get(path, **kwargs)
        
        return response


class Companies(BaseResource):
    """Get information about movie companies.

    See: http://docs.themoviedb.apiary.io/#companies
    """
    BASE_PATH = 'company/'
    URLS = {
        'info': '{id}',
        'movies': '{id}/movies',
    }

    async def info(self, company_id, **kwargs):
        """Get the basic information about a movie company.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fcompany%2F%7Bid%7D

        :param append_to_response: (optional) Any Collections method names. E.g.
                                   'movies'
        """
        path = self._get_path('info').format(id=company_id)

        response = await self._get(path, **kwargs)
        
        return response

    async def movies(self, company_id, **kwargs):
        """Get the list of movies associated with a particular company.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fcompany%2F%7Bid%7D%2Fmovies

        :param page: (optional) Result page number, e.g. 3.
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param append_to_response: (optional) Any Collections method names. E.g.
                                   'info'
        """
        path = self._get_path('movies').format(id=company_id)

        response = await self._get(path, **kwargs)
        
        return response


class Reviews(BaseResource):
    """Get information about reviews.

    See: http://docs.themoviedb.apiary.io/#reviews
    """
    BASE_PATH = 'review/'
    URLS = {
        'info': '{id}',
    }

    async def info(self, review_id, **kwargs):
        """Get the full details of a review by ID.

        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Freview%2F%7Bid%7D
        """
        path = self._get_path('info').format(id=review_id)

        response = await self._get(path, **kwargs)
        
        return response


import logging
import aiohttp

from .exceptions import TMDbException

from .resources.discover import Discover
from .resources.genres import Genres
from .resources.search import Search
from .resources.tv import TV
from .resources.movies import Movies, Collections, Companies, Reviews

logging.basicConfig(level=logging.INFO)
_log = logging.getLogger(__name__)

DEFAULT_OPTIONS = {
    "language": "en",
    "cache": True
}

class TMDb(object):
    def __init__(self, api_key, **options):
        options = {
            **DEFAULT_OPTIONS,
            **options
        }

        self._base = 'http://api.themoviedb.org/3'
        self._remaining = 40
        self._reset = None
        self.api_key = api_key
        self.cache = options['cache']
        self.language = options['language']

        # Resources
        self.discover = Discover(self)
        self.genres = Genres(self)
        self.search = Search(self)
        self.tv = TV(self)
        self.movies = Movies(self)
        self.collections = Collections(self)
        self.companies = Companies(self)
        self.reviews = Reviews(self)

    async def _request(self, url, append_to_response="", method="get", params={}, data={}):
        if self.api_key is None or self.api_key == '':
            raise TMDbException("No API key found.")

        params = {
            "api_key": self.api_key,
            "language": self.language,
            **params
        }

        async with aiohttp.ClientSession() as session:
            async with getattr(session, method)(url, params=params) as resp:
                if 'X-RateLimit-Remaining' in resp.headers:
                    _remaining = int(headers['X-RateLimit-Remaining'])

                if 'X-RateLimit-Reset' in resp.headers:
                    _reset = int(headers['X-RateLimit-Reset'])

                data = await resp.json(content_type=None)

                if resp.status >= 300:
                    raise TMDbException(data)

                return data

        await session.close()



from .base import BaseResource


class Genres(BaseResource):
    """ 
    See: http://docs.themoviedb.apiary.io/#genres
    """
    BASE_PATH = 'genre/'
    URLS = {
        'movie_list': 'movie/list',
        'tv_list': 'tv/list',
        'movies': '{id}/movies',
    }

    async def movie_list(self, **kwargs):
        """Get the list of genres for movies.
        """
        path = self._get_path('movie_list')

        response = await self._get(path, **kwargs)
        
        return response

    async def tv_list(self, **kwargs):
        """Get the list of genres for TV shows.
        """
        path = self._get_path('tv_list')

        response = await self._get(path, **kwargs)
        
        return response

    async def movies(self, **kwargs):
        """Get the list of movies for a particular genre by id.
        By default, only movies with 10 or more votes are included.
        """
        path = self._get_path('movies').format(id=kwargs['id'])

        response = await self._get(path, **kwargs)
        
        return response



from .base import BaseResource


class Search(BaseResource):
    """Search movies, TV shows, collections, persons, lists, companies and
    keywords.
    See: http://docs.themoviedb.apiary.io/#search
    """
    BASE_PATH = 'search/'
    URLS = {
        'movie': 'movie',
        'collection': 'collection',
        'tv': 'tv',
        'person': 'person',
        'list': 'list',
        'company': 'company',
        'keyword': 'keyword',
        'multi': 'multi',
    }

    async def movies(self, query, **kwargs):
        """Search for movies by title.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fsearch%2Fmovie
        """
        path = self._get_path('movie')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        return response

    async def collections(self, query, **kwargs):
        """Search for collections by name.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fsearch%2Fcollection
        """
        path = self._get_path('collection')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        
        return response

    async def tv(self, query, **kwargs):
        """Search for TV shows by title.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fsearch%2Ftv
        """
        path = self._get_path('tv')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        
        return response

    async def persons(self, query, **kwargs):
        """Search for people by name.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fsearch%2Fperson
        """
        path = self._get_path('person')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        
        return response

    async def companies(self, query, **kwargs):
        """Search for companies by name.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fsearch%2Fcompany
        """
        path = self._get_path('company')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        
        return response

    async def keywords(self, query, **kwargs):
        """Search for keywords by name.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fsearch%2Fkeyword
        """
        path = self._get_path('keyword')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        
        return response

    async def multi(self, query, **kwargs):
        """Search the movie, tv show and person collections with a single query.
        Each item returned in the result array has a media_type field that maps to
        either movie, tv or person. Each mapped result is the same response you
        would get from each independent search.
        """
        path = self._get_path('multi')
        kwargs.update({'query': query})

        response = await self._get(path, **kwargs)
        
        return response


from .base import BaseResource


class TV(BaseResource):
    """Interact with The Movie Database TV endpoints
    See: http://docs.themoviedb.apiary.io/#tv
    """
    BASE_PATH = 'tv/'
    URLS = {
        'info': '{id}',
        'credits': '{id}/credits',
        'external_ids': '{id}/external_ids',
        'images': '{id}/images',
        'translations': '{id}/translations',
        'top_rated': 'top_rated',
        'popular': 'popular',
    }

    async def info(self, **kwargs):
        """Get the primary information about a TV series by id.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D
        """
        path = self._get_path('info').format(id=kwargs['id'])

        response = await self._get(path, **kwargs)
        
        return response

    async def credits(self, **kwargs):
        """Get the cast & crew information about a TV series.
        Just like on the TMDB website, this information is pulled from the last
        season of the series.
        """
        path = self._get_id_path('credits')

        response = await self._get(path, **kwargs)
        
        return response

    async def external_ids(self, **kwargs):
        """Get the external ids for a TV series.
        E.g. IMDB, TVRage, Freebase.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Fexternal_ids
        """
        path = self._get_id_path('external_ids')

        response = await self._get(path, **kwargs)
        
        return response

    async def images(self, **kwargs):
        """Get the images (posters and backdrops) for a TV series.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Fimages
        """
        path = self._get_id_path('images')

        response = await self._get(path, **kwargs)
        
        return response

    async def translations(self, **kwargs):
        """Get the list of translations that exist for a TV series.
        These translations cascade down to the episode level.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Ftranslations
        """
        path = self._get_id_path('translations')

        response = await self._get(path, **kwargs)
        
        return response

    async def top_rated(self, **kwargs):
        """Get the list of top rated TV shows.
        By default, this list will only include TV shows that have 2 or more
        votes. This list refreshes every day.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2Ftop_rated
        """
        path = self._get_id_path('top_rated')

        response = await self._get(path, **kwargs)
        
        return response

    async def popular(self, **kwargs):
        """Get the list of popular TV shows.
        This list refreshes every day.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2Fpopular
        """
        path = self._get_id_path('popular')

        response = await self._get(path, **kwargs)
        
        return response


class TVSeasons(BaseResource):
    """Interact with The Movie Database TV season endpoints
    See: http://docs.themoviedb.apiary.io/#tvseasons
    """
    BASE_PATH = 'tv/{id}/season/'
    URLS = {
        'info': '{season}',
        'credits': '{season}/credits',
        'external_ids': '{season}/external_ids',
        'images': '{season}/images'
    }


    async def _get_season_id_path(self, key, id, season):
        return self._get_path(key).format(id=id, season=season_number)

    async def info(self, **kwargs):
        """Get the primary information about a TV season.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Fseason%2F%7Bseason_number%7D
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_season_id_path('info')

        response = await self._get(path, **kwargs)
        
        return response

    async def credits(self, **kwargs):
        """Get the cast & crew credits for a TV season.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Fseason%2F%7Bseason_number%7D%2Fcredits
        """
        path = self._get_season_id_path('credits')

        response = await self._get(path, **kwargs)
        
        return response

    async def external_ids(self, **kwargs):
        """Get the external ids for a TV season.
        E.g. IMDB, TVRage, Freebase.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Fseason%2F%7Bseason_number%7D%2Fexternal_ids
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        """
        id_ = kwargs.pop('id')
        season = kwargs.pop('season')
        path = self._get_season_id_path('external_ids', id_, season)

        response = await self._get(path, **kwargs)
        
        return response

    async def images(self, **kwargs):
        """Get the images (posters) for a TV season.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Ftv%2F%7Bid%7D%2Fseason%2F%7Bseason_number%7D%2Fimages
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param include_image_language: (optional) Comma separated ISO 639-1 codes,
                                       e.g. 'en, es'.
        """
        path = self._get_season_id_path('images')

        response = await self._get(path, **kwargs)
        
        return response


class TVEpisodes(BaseResource):
    """Interact with The Movie Database TV episode endpoints
    See: http://docs.themoviedb.apiary.io/#tvepisodes
    """
    BASE_PATH = 'tv/{id}/season/{season}/episode/'
    URLS = {
        'info': '{episode}',
        'credits': '{episode}/credits',
        'external_ids': '{episode}/external_ids',
        'images': '{episode}/images'
    }

    async def __init__(self, id, season_number, episode_number):
        super(TVEpisodes, self).__init__()
        self.id = id
        self.season_number = season_number
        self.episode_number = episode_number

    async def _get_episode_id_path(self, key):
        return self._get_path(key).format(id=self.id, season=self.season_number,
                                          episode=self.episode_number)

    # http://preview.tinyurl.com/get-tv-id-episode
    # optional parameter: language
    async def info(self, **kwargs):
        """Get the basic movie information for a specific movie id.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        :param append_to_response: (optional) Any Movies method names. E.g.
                                   'credits, images'
        """
        path = self._get_episode_id_path('info')

        response = await self._get(path, **kwargs)
        
        return response

    # http://preview.tinyurl.com/get-tv-id-episode-credits
    # optional parameter: language
    async def credits(self, **kwargs):
        """Get the basic movie information for a specific movie id.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D
        """
        path = self._get_episode_id_path('credits')

        response = await self._get(path, **kwargs)
        
        return response

    # http://tinyurl.com/get-tv-id-episode-external-ids
    # optional parameter: language
    async def external_ids(self, **kwargs):
        """Get the basic movie information for a specific movie id.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        """
        path = self._get_episode_id_path('external_ids')

        response = await self._get(path, **kwargs)
        
        return response

    async def images(self, **kwargs):
        """Get the basic movie information for a specific movie id.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fmovie%2F%7Bid%7D
        :param language: (optional) ISO 639-1 code, e.g. 'de'. For a list of
                         639-1 codes, see http://en.wikipedia.org/wiki/ISO_639-1
        """
        path = self._get_episode_id_path('images')

        response = await self._get(path, **kwargs)
        
        return response


class Networks(BaseResource):
    """Information about TV networks.
    See: http://docs.themoviedb.apiary.io/#networks
    """
    BASE_PATH = 'network/'
    URLS = {
        'info': '{id}'
    }

    async def info(self, network_id):
        """Get the basic information about a TV network.
        You can use this ID to search for TV shows with the discover.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fnetwork%2F%7Bid%7D
        """
        path = self._get_path('info').format(id=network_id)

        response = self._get(path)
        
        return response


from .base import BaseResource


class Discover(BaseResource):
    """Discover
    See: http://docs.themoviedb.apiary.io/#discover
    """
    BASE_PATH = 'discover/'
    URLS = {
        'movie': 'movie',
        'tv': 'tv',
    }

    def _replace_underscore(self, dictionary):
        """Replace the last '_' (underscore) with a '.'' (dot)."""
        for key in dictionary:
            if 'gte' in key or 'lte' in key:
                new_key = '.'.join(key.rsplit('_', 1))
                dictionary[new_key] = dictionary.pop(key)


    async def movie(self, **kwargs):
        """Discover movies by different types of data like average rating,
        number of votes, genres and certifications.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fdiscover%2Fmovie
        """
        
        path = self._get_path('movie')
        self._replace_underscore(kwargs)
        response = await self._get(path, **kwargs)

        return response

    async def tv(self, **kwargs):
        """Discover TV shows by different types of data like average rating,
        number of votes, genres, the network they aired on and air dates.
        TMDB doc: http://docs.themoviedb.apiary.io/#get-%2F3%2Fdiscover%2Ftv
        """
        
        path = self._get_path('tv')
        self._replace_underscore(kwargs)
        response = await self._get(path, **kwargs)
        return response



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aiotmdb']

package_data = \
{'': ['*'], 'aiotmdb': ['resources/*']}

install_requires = \
['aiohttp']

extras_require = \
{'test': ['pytest >=2.7.3']}

setup(name='aiotmdb',
      version='0.3',
      description='Asyncio library for TMDb API',
      author='Fabrizio Arzeni',
      author_email='fabrizio.arz@gmail.com',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )




class BaseResource(object):
    BASE_PATH = ''
    URLS = {}

    def __init__(self, tmdb):
        self.client = tmdb

    def _get_path(self, key):
        return self.BASE_PATH + self.URLS[key]

    def _get_id_path(self, key):
        return self._get_path(key).format(id=self.id)

    def _get_complete_url(self, path):
        return '{base}/{path}'.format(base=self.client._base, path=path)

    async def _get(self, path, **params):
        url = self._get_complete_url(path)
        return await self.client._request(url, method="get", params=params)

    async def _post(self, path, data, **params):
        url = self._get_complete_url(path)
        return await self.client._request(url, method="post", params=params, data=data)
